Frog = Object:extend()

function Frog:new(x, y, health)
    self.x = x
    self.y = x
    self.width = 40
    self.height = 40
    self.speedX = 50
    self.speedY = -100
    self.scaleX = 1
    self.moveto1 = x + 50
    self.moveto2 = x - 50
    self.phase1 = false
    self.phase2 = false
    self.dirX = -1
    self.dead = false
    
    self.sprite = love.graphics.newImage('frog.png')
    self.grid = anim8.newGrid(35, 32, self.sprite:getWidth(), self.sprite:getHeight())
    self.animations = {}
    self.animations.idle = anim8.newAnimation(self.grid('1-4', 1), 0.2)
    self.animations.jumpup = anim8.newAnimation(self.grid(6, 1), 0.2)
    self.animations.jumpdown = anim8.newAnimation(self.grid(7, 1), 0.2)
    self.anim = self.animations.idle
    
    self.idleState = true
    self.jumpupState = false
    self.jumpdownState = false
    
    self.collider = world:newRectangleCollider(self.x, self.y, 20, 18)
    --self.collider = world:newCircleCollider(self.x, self.y, 10)
    self.collider:setCollisionClass('Frog')
    self.collider:setObject(self)
    
    self.jumpingReload = 200
    self.health = health
end

function Frog:update(dt)
    self.anim:update(dt)
    
    
    
    if self.jumpingReload > 0 then
        self.jumpingReload = self.jumpingReload - 1
    end
    
    xVel,yVel = self.collider:getLinearVelocity()
    
    if yVel > 4 then
        self.jumpdownState = true
        self.jumpupState = false
        self.idle = false
    elseif yVel < -4 then
        self.jumpupState = true
        self.jumpdownState = false
        self.idle = false
    else
        self.jumpupState = false
        self.jumpdownState = false
        self.idle = true
    end
    
    
    --claim
    
    self.x = self.collider:getX()
    self.y = self.collider:getY()
    
    if self.x < self.moveto2 then
        self.phase1 = false
        self.phase2 = true
    elseif self.x > self.moveto1 then
        self.phase1 = true
        self.phase2 = false
    end
    if self.phase1 then
        self.scaleX = 1
        self.dirX = -1
    elseif self.phase2 then
        self.scaleX = -1
        self.dirX = 1
    end
    
    if self.x == moveto2 then
        self.phase1 = false
        self.phase2 = true
    elseif self.x == moveto1 then
        self.phase1 = true
        self.phase2 = false
    end
    
    if self.jumpingReload < 1 then
        self.randX = love.math.random(1, 3)
        self.randY = love.math.random(1, 3)
        if self.randX == 1 then
            self.speedX = 30
        elseif self.randX == 2 then
            self.speedX = 40
        else
            self.speedX = 50
        end
        
        if self.randY == 1 then
            self.speedY = -100
        elseif self.randY == 2 then
            self.speedY = -125
        else
            self.speedY = -150
        end
        self.collider:applyLinearImpulse(self.speedX * self.dirX, self.speedY)
        self.jumpingReload = 200
    end
    
    if self.jumpupState then
        self.anim = self.animations.jumpup
    elseif self.jumpdownState then
        self.anim = self.animations.jumpdown
    else
        self.anim = self.animations.idle
    end
    
    if self.health <= 0 then
            self:boom()
    end
end

function Frog:boom()
    for k, v in ipairs(listOfFrog) do
	      if v == self then 
            table.insert(listOfExplo, EnemyExplosion(self.collider:getX(), self.collider:getY(), "eneExplo", 35))
            self.collider:destroy()
            table.remove(listOfFrog, k) 
            break
        end
    end
    --self.collider:destroy()
    --listOfFrog:pullval(self)
end

function Frog:draw()
    self.anim:draw(self.sprite, self.x, self.y, nil, self.scaleX, 1, 16, 17.5)
end