Player = Object:extend()

function Player:new()
    self.width = 32
    self.height = 32
    self.x = 300
    self.y = 300
    self.speedX = 0
    
    self.scaleX = 1
    self.jumpupState = false
    self.jumpdownState = false
    self.idleState = true
    self.runleftState = false
    self.runrightState = false
    self.hurtState = false
    self.hurtReload = 50
    self.ouchX = 25
    self.cherryReload = 20
    self.stompReload = 20
    
    self.sprite = love.graphics.newImage('foxsheet.png')
    self.grid = anim8.newGrid(self.width, self.height, 192, 192)
    self.animations = {}
    self.animations.idle = anim8.newAnimation(self.grid('1-4',5), 0.1)
    self.animations.run = anim8.newAnimation(self.grid('1-6',1), 0.075)
    self.animations.jumpup = anim8.newAnimation(self.grid(1,6), 0.35)
    self.animations.jumpdown = anim8.newAnimation(self.grid(2,6), 0.35)
    self.animations.crouch = anim8.newAnimation(self.grid('1-2',3), 0.1)
    self.animations.hurt = anim8.newAnimation(self.grid('1-2',4), 0.1)
    self.animations.climb = anim8.newAnimation(self.grid('1-3',2), 0.1)
    self.anim = self.animations.idle
    
    
    
    self.collider = world:newRectangleCollider(self.x, self.y, 15, 15)
    --self.collider = world:newCircleCollider(self.x, self.y, 10)
    self.collider:setCollisionClass('Player')
    self.collider:setObject(self)
    
    self.jumping = false
    self.dir = "right"
end

function Player:update(dt)
    self.anim:update(dt)
    
    if self.collider:enter('Platform') or self.collider:enter('Frog') then
        self.jumping = false
    end
    
    if self.collider:enter('Frog') then    
        local collision_data = self.collider:getEnterCollisionData('Frog')
        local enemy = collision_data.collider:getObject()
        if enemy.collider:getY() > self.collider:getY() then
            self.collider:applyLinearImpulse(0, -50)
            --print("ouch")
            enemy.health = enemy.health - 1
            
            
        else
            if self.hurtState == false then
                self.hurtState = true
                if self.collider:getX() < enemy.collider:getX() then
                    self.ouchX = -25
                else
                    self.ouchX = 25
                end
                self.collider:applyLinearImpulse(self.ouchX, -100)
            end
        end
        
    end
    
    if self.hurtState then
        self.hurtReload = self.hurtReload - 1
    end
    
    if self.hurtReload <= 0 then
        self.hurtState = false
        self.hurtReload = 50
    end
    
    if love.keyboard.isDown("up") and not self.jumping then
        self.collider:applyLinearImpulse(0, -100)
        self.jumping = true
    end
    
    x,y = self.collider:getLinearVelocity()
    
    if y>3 then
        self.jumpdownState = true
        self.jumpupState = false
    elseif y<-3 then
        self.jumpupState = true
        self.jumpdownState = false
    else
        self.jumpupState = false
        self.jumpdownState = false
    end
    
    if love.keyboard.isDown("right") then
        self.speedX = 7
        self.runrightState = true
        self.dir = "right"
    else
        self.runrightState = false
    end
    
    if love.keyboard.isDown("left") then
        self.speedX = -7
        self.runleftState = true
        self.dir = "left"
    else
        self.runleftState = false
    end
    
    if not love.keyboard.isDown("left") and not love.keyboard.isDown("right") then
        self.speedX = 0
        self.idleState = true
    else
        self.idleState = false
    end
    
    
    if (math.abs(self.collider:getLinearVelocity()) < 60) then
        self.collider:applyLinearImpulse(self.speedX, 0)
    end
    
    --animations here
    if self.hurtState then
        self.anim = self.animations.hurt
    else
    if self.jumpupState then
        self.anim = self.animations.jumpup
    elseif self.jumpdownState then
        self.anim = self.animations.jumpdown
    else
        if self.runrightState then
            self.anim = self.animations.run
            self.scaleX = 1
        elseif self.runleftState then
            self.anim = self.animations.run
            self.scaleX = -1
        else
            self.anim = self.animations.idle
        end
    end
    
    end
    
    self.x = self.collider:getX()
    self.y = self.collider:getY()
    self.cherryReload = self.cherryReload - 1
    self.stompReload = self.stompReload - 1
    if love.keyboard.isDown("space") and self.cherryReload <=0 then
        if self.dir == "right" then
            table.insert(listOfDamageObject, DamageObject(self.x, self.y, 20, 20, 7, "cherrybomb", 100, 50))
        else
            table.insert(listOfDamageObject, DamageObject(self.x, self.y, 20, 20, 7, "cherrybomb", 100, -50))
        end
        self.cherryReload = 20
    end
      
    if love.keyboard.isDown("down") and self.stompReload <= 0 then
        table.insert(listOfDamageObject, DamageObject(self.x-20, self.y, 40, 2, nil, "stomp", 30, 0))
        self.stompReload = 20
    end
    
end

function Player:draw()
    self.anim:draw(self.sprite, self.x, self.y, nil, self.scaleX, 1, 16, 16)
end